@@ -94,7 +94,7 @@ ModvegeSite = R6Class(
   #-Public-attributes-------------------------------------------------------------
 #' @field time_step Used time step in the model in days (untested).
       time_step = 1.,
-#' @field state_variable_namse Vector containing the names of the model's 
+#' @field state_variable_names Vector containing the names of the model's 
 #'   state variables.
       state_variable_names = NULL,
 #' @field n_state_variables Number of state variables.
@@ -167,6 +167,10 @@ ModvegeSite = R6Class(
 #' @field management A [ManagementData] object. If its `is_empty` field is `TRUE`, 
 #'   the autocut routine will be employed.
       management = NULL,
+#' @field stubble_height float. Minimum height the grass can assume. The 
+#'    biomass will not fall below that height. This can and should therefore 
+#'    be smaller than `self$cut_height`.
+      stubble_heigt = 0.01,
 
   #-Public-methods----------------------------------------------------------------
 
@@ -351,6 +355,9 @@ ModvegeSite = R6Class(
       run = function(year, weather, management) {
         logger("Start of ModvegeSite$run()", level = TRACE)
         self$weather = weather
+        # Infer days of this year from weather data
+        self$days_per_year = weather$ndays
+        logger(sprintf("days_per_year: %s", self$days_per_year), level = DEBUG)
         self$management = management
         self$year = year
         private$initialize_state_variables()
@@ -463,14 +470,49 @@ ModvegeSite = R6Class(
         self$parameters$set_parameters(params)
       },
 
-      #' @description Create an overview plot
+      #' @description Create an overview plot for 16 state variables.
+      #'
+      #' Creates a simple base R plot showing the temporal evolution of 16 
+      #' modeled state variables.
+      #'
+      #' Can only be sensibly run *after* a simulation has been carried out, 
+      #' i.e. after this instance's `run()` method has been called.
+      #'
+      #' @param ... Further arguments are discarded.
+      #' @return NULL Creates a plot of the result in the active device.
+      #'
+      plot = function(...) {
+        oldpar = par(no.readonly = TRUE)
+        on.exit(par(oldpar))
+        par(mfrow = c(4, 4))
+        vars_to_plot = c("BM", "cBM", "hvBM", "dBM",
+                         "ENV", "ENVfT", "ENVfW", "ENVfPAR",
+                         "WR", "AET", "LAI", "LAIGV",
+                         "PGRO", "REP", "GRO", "ST")
+        n_vars_to_plot = length(vars_to_plot)
+        # Plot the first variable with a title indicating site name and year.
+        self$plot_var(vars_to_plot[1])
+        # Plot the remaining variables with the variable name as title.
+        for (i in 2:n_vars_to_plot) {
+          var = vars_to_plot[i]
+          self$plot_var(var, main = var)
+        }
+      },
+
+      #' @description Create an overview plot for biomass.
       #'
       #' Creates a simple base R plot showing the BM with cutting events and,
       #' if applicable, target biomass, dBM, cBM and hvBM.
+      #' Can only be sensibly run *after* a simulation has been carried out, 
+      #' i.e. after this instance's `run()` method has been called.
       #'
-      #' @return None Creates a plot of the result.
+      #' @param smooth_interval Int. Number of days over which the variable 
+      #'   `dBM` is smoothened.
+      #' @param ... Further arguments are discarded.
+      #' @return NULL Creates a plot of the result in the active device.
       #'
-      plot = function() {
+      plot_bm = function(smooth_interval = 28, ...) {
+        private$check_if_simulation_has_run()
         oldpar = par(no.readonly = TRUE)
         on.exit(par(oldpar))
         par(mfrow = c(2, 2))
@@ -483,12 +525,108 @@ ModvegeSite = R6Class(
           lines(self$target_biomass, col = "grey")
         }
         # dBM, cBM, hBM
-        plot(box_smooth(self$dBM, 28), type = "l", xlab = xlab, 
-             ylab = "smoothened dBM (kg / ha)")
+        plot(box_smooth(self$dBM, smooth_interval), type = "l", 
+             xlab = xlab, ylab = "smoothened dBM (kg / ha)")
         plot(self$cBM, type = "l", xlab = xlab, ylab = "cBM (kg / ha)")
         plot(self$hvBM, type = "l", xlab = xlab, ylab = "hvBM (kg / ha)")
-      }
+      },
 
+      #' @description Create an overview plot of limiting factors.
+      #'
+      #' Creates a simple base R plot showing the different environmental 
+      #' limitation functions over time.
+      #' Can only be sensibly run *after* a simulation has been carried out, 
+      #' i.e. after this instance's `run()` method has been called.
+      #'
+      #' @param ... Further arguments are discarded.
+      #' @return NULL Creates a plot of the result in the active device.
+      #'
+      plot_limitations = function(...) {
+        oldpar = par(no.readonly = TRUE)
+        on.exit(par(oldpar))
+        par(mfrow = c(2, 2))
+        ylim = c(0, 1)
+        self$plot_var("ENV", ylim = ylim)
+        self$plot_var("ENVfW", ylim = ylim, main = "limitation functions")
+        self$plot_var("ENVfT", ylim = ylim, main = "limitation functions")
+        self$plot_var("ENVfPAR", ylim = ylim, main = "limitation functions")
+      },
+
+      #' @description Create an overview plot of the water balance.
+      #'
+      #' Creates a simple base R plot showing different variables pertaining to
+      #' the water balance, namely water reserves *WR*, actual 
+      #' evapotranspiration *AET*, leaf area index *LAI* and LAI of the green 
+      #' vegetative compartment *LAIGV*. 
+      #'
+      #' Can only be sensibly run *after* a simulation has been carried out, 
+      #' i.e. after this instance's `run()` method has been called.
+      #'
+      #' @param ... Further arguments are discarded.
+      #' @return NULL Creates a plot of the result in the active device.
+      #'
+      plot_water = function(...) {
+        oldpar = par(no.readonly = TRUE)
+        on.exit(par(oldpar))
+        par(mfrow = c(2, 2))
+        self$plot_var("WR")
+        self$plot_var("AET", main = "water balance")
+        self$plot_var("LAI", main = "")
+        self$plot_var("LAIGV", main = "")
+      },
+
+      #' @description Create an overview plot of growth dynamics.
+      #'
+      #' Creates a simple base R plot showing different variables pertaining to
+      #' the growth dynamics, namely potential growth *PGRO*, effective 
+      #' growth *GRO*, the reproductive function *REP* and the temperature 
+      #' sum *ST*. 
+      #'
+      #' Can only be sensibly run *after* a simulation has been carried out, 
+      #' i.e. after this instance's `run()` method has been called.
+      #'
+      #' @param ... Further arguments are discarded.
+      #' @return NULL Creates a plot of the result in the active device.
+      #'
+      plot_growth = function(...) {
+        oldpar = par(no.readonly = TRUE)
+        on.exit(par(oldpar))
+        par(mfrow = c(2, 2))
+        self$plot_var("PGRO")
+        self$plot_var("REP", main = "growth dynamics")
+        self$plot_var("GRO", main = "")
+        self$plot_var("ST", main = "")
+      },
+
+      #' @description Plot the temporal evolution of a modeled state variable.
+      #'
+      #' @param var String. Name of the state variable to plot.
+      #' @param ... Further arguments are passed to the base [plot()] function.
+      #' @return None, but plots to the current device.
+      #'
+      plot_var = function(var, ...) {
+        private$check_if_simulation_has_run()
+        # Check for valid input.
+        if (!var %in% self$state_variable_names) {
+          message = "Variable `%s` is invalid for plotting. Use one of:\n%s"
+          vars = paste(self$state_variable_names, collapse = ", ")
+          warning(sprintf(message, var, vars))
+          return()
+        }
+        arguments = list(...)
+        # Set default values
+        if (!"type" %in% names(arguments)) {
+          arguments[["type"]] = "l"
+        }
+        if (!"main" %in% names(arguments)) {
+          arguments[["main"]] = paste(self$site_name, self$run_name, self$year)
+        }
+        arguments[["xlab"]] = "DOY"
+        arguments[["ylab"]] = private$ylabels[[var]]
+        arguments[["x"]] = self$weather$DOY
+        arguments[["y"]] = self[[var]]
+        do.call(plot, arguments)
+      }
     )
   ), # End of public attributes
 
@@ -498,6 +636,39 @@ ModvegeSite = R6Class(
     vars_to_exclude = c("OMDDV", "OMDDR"),
     current_DOY = 1,
     REP_ON = NULL,
+    # List of labels to use for different variables when plotting.
+    ylabels  =  list(AgeGV = "AgeGV (degree days)",
+                   AgeGR = "AgeGR (degree days)",
+                   AgeDV = "AgeDV (degree days)",
+                   AgeDR = "AgeDR (degree days)",
+                   BMGV = "BMGV (kg/ha)",
+                   BMGR = "BMGR (kg/ha)",
+                   BMDV = "BMDV (kg/ha)",
+                   BMDR = "BMDR (kg/ha)",
+                   BM = "standing biomass BM (kg/ha)",
+                   BMG = "standing green biomass BMG (kg/ha)",
+                   cBM = "cumulative biomass cBM (kg/ha)",
+                   dBM = "daily biomass growth dBM (kg/ha/d)",
+                   hvBM = "total harvested biomass growth hvBM (kg/ha)",
+                   OMD = "organic matter digestibility OMD (kg/kg)",
+                   OMDG = "OMDG (kg/kg)",
+                   OMDGV = "OMDGV (kg/kg)",
+                   OMDGR = "OMDGR (kg/kg)",
+                   OMDDV = "OMDDV (kg/kg)",
+                   OMDDR = "OMDDR (kg/kg)",
+                   ST = "temperature sum (degree days)",
+                   REP = "reproductive function REP",
+                   GRO = "daily growth (kg/ha/d)",
+                   PGRO = "potential daily growth (kg/ha/d)",
+                   LAI = "leaf area index LAI",
+                   LAIGV = "LAIGV",
+                   AET = "actual evapotranspiration AET (mm)",
+                   WR = "water reserves WR (mm)",
+                   ENV = "environmental limitation ENV",
+                   ENVfPAR = "radiation limitation ENVfPAR",
+                   ENVfT = "temperature limitation ENVfT",
+                   ENVfW = "water limitation ENVfW"
+                   ),
 
     #-Private-methods-----------------------------------------------------------
 
@@ -527,9 +698,6 @@ ModvegeSite = R6Class(
       self[["OMDGV"]][1] = P$OMDGV0
       self[["OMDGR"]][1] = P$OMDGR0
 
-      # (minSEA + maxSEA)/2 = 1
-      self$parameters[["minSEA"]] = 2 - P[["maxSEA"]]
-
       # Management:
       # Initialize a pointer that indicates whether there has been a cut 
       # during reproductive growth. In this case, reproductive growth is
@@ -654,7 +822,8 @@ ModvegeSite = R6Class(
         PETeff = PETmn
       } else {
         # Less evapotranspiration when there is precipitation.
-        PETeff = ifelse(W$PP[j] > 1, 0.7 * W$PET[j], W$PET[j])
+        PETeff = P$crop_coefficient * W$PET[j]
+        PETeff = ifelse(W$PP[j] > 1, 0.7 * PETeff, PETeff)
       }
       PETeff = PETeff * fCO2_transpiration_mod(W$aCO2)
       PTr = PETeff * (1. - exp(-0.6 * LAI.ET))
@@ -672,7 +841,7 @@ ModvegeSite = R6Class(
       # Environmental constraints.
       self$ENVfPAR[j] = fPAR(W$PAR[j])
       self$ENVfT[j]   = fT(W$Ta[j], P$T0, P$T1, P$T2)
-      self$ENVfW[j]   = fW(self$WR[j] / P$WHC, W$PET[j])
+      self$ENVfW[j]   = fW(self$WR[j] / P$WHC, PETeff)
       self$ENV[j]     = self$ENVfPAR[j] * self$ENVfT[j] * self$ENVfW[j]
 
       if (j < self$j_start_of_growing_season) {
@@ -761,23 +930,23 @@ ModvegeSite = R6Class(
         self$SENGV = P$KGV * self$BMGVp * T_average * fAgeGV
         self$SENGR = P$KGR * self$BMGRp * T_average * fAgeGR
       } else if (T_average > 0.) {
-        # Too cold for photosynthesis, but above freezing
+        # Too cold for photosynthesis, but above freezing.
         self$SENGV = 0.
         self$SENGR = 0.
       } else {
-        # Below 0 temperature-> freezing damage
-        self$SENGV = -P$KGV * self$BMGVp * T_average
-        self$SENGR = -P$KGR * self$BMGRp * T_average
+        # Below 0 temperature-> freezing damage.
+        self$SENGV = P$KGV * self$BMGVp * abs(T_average)
+        self$SENGR = P$KGR * self$BMGRp * abs(T_average)
       }
 
       # Put a cap on senescence.
       # :NOTE: The following two lines were not part of the original model 
       # formulation.
-      if (abs(self$SENGV) > 0.7 * abs(self$GROGV)) { 
-        self$SENGV = 0.7 * self$GROGV 
+      if (abs(self$SENGV) > P$senescence_cap * abs(self$GROGV)) { 
+        self$SENGV = P$senescence_cap * self$GROGV 
       }
-      if (abs(self$SENGR) > 0.7 * abs(self$GROGR)) { 
-        self$SENGR = 0.7 * self$GROGR 
+      if (abs(self$SENGR) > P$senescence_cap * abs(self$GROGR)) { 
+        self$SENGR = P$senescence_cap * self$GROGR 
       }
      
       # Calculate ageing for compartments DV & DR.
@@ -834,12 +1003,13 @@ ModvegeSite = R6Class(
     update_biomass = function() {
       j = private$current_DOY
       P = self$parameters
+      # SEN is always >= 0
       dBMGV = self$GROGV - self$SENGV
       self$BMGV[j] = max(self$BMGVp + dBMGV * self$time_step, 
-                         0.01 * 10. * P$BDGV)
+                         self$stubble_height * 10. * P$BDGV)
       dBMGR = self$GROGR - self$SENGR
       self$BMGR[j] = max(self$BMGRp + dBMGR * self$time_step, 
-                         0.01 * 10. * P$BDGR)
+                         self$stubble_height * 10. * P$BDGR)
       
       dBMDV = (1. - P$sigmaGV) * self$SENGV - self$ABSDV
       self$BMDV[j] = max(self$BMDVp + dBMDV * self$time_step, 0.)
@@ -959,7 +1129,34 @@ ModvegeSite = R6Class(
                        header, self$year, self$site_name, self$run_name)
       logger("End of ModVegeSite$make_header()", level = TRACE)
       return(header)
-    }
+    },
 
+    ## Plot only works, after the simulation has run.
+    ##
+    check_if_simulation_has_run = function() {
+      if (private$current_DOY == 1) {
+        warning("Cannot plot results because simulation has not yet been run.")
+        return()
+      }
+    }
   ) # End of private attributes
 )
+
+## S3 dispatch methods
+
+#' Plot ModVege simulation result overview
+#'
+#' This wraps the `ModvegeSite` instance's `plot()` method.
+#'
+#' @param x A [ModvegeSite] instance.
+#' @param ... Arguments are passed on to [ModvegeSite]`$plot()`.
+#' @return NULL, but plots to the active device.
+#'
+#' @seealso The different `[modvegeSite]$plot_XXX()` methods.
+#'
+#' @md
+#' @export
+plot.ModvegeSite = function(x, ...) {
+  x$plot(...)
+}
+
